@@ -126,6 +126,11 @@ def _determine_linesep(
         index_url=redact_auth_from_url(_get_default_option("index_url"))
     ),
 )
+@click.option(
+    "--no-index",
+    is_flag=True,
+    help="Ignore package index (only looking at --find-links URLs instead).",
+)
 @click.option(
     "--extra-index-url",
     multiple=True,
@@ -302,6 +307,7 @@ def cli(
     all_extras: bool,
     find_links: tuple[str, ...],
     index_url: str,
+    no_index: bool,
     extra_index_url: tuple[str, ...],
     cert: str | None,
     client_cert: str | None,
@@ -392,6 +398,8 @@ def cli(
         pip_args.extend(["-f", link])
     if index_url:
         pip_args.extend(["-i", index_url])
+    if no_index:
+        pip_args.extend(["--no-index"])
     for extra_index in extra_index_url:
         pip_args.extend(["--extra-index-url", extra_index])
     if cert:
@@ -417,10 +425,7 @@ def cli(
         key_from_ireq(install_req): install_req for install_req in upgrade_reqs_gen
     }
 
-    existing_pins_to_upgrade = set()
-
-    # Exclude packages from --upgrade-package/-P from the existing
-    # constraints, and separately gather pins to be upgraded
+    # Exclude packages from --upgrade-package/-P from the existing constraints
     existing_pins = {}
 
     # Proxy with a LocalRequirementsRepository if --upgrade is not specified
@@ -438,9 +443,7 @@ def cli(
 
         for ireq in filter(is_pinned_requirement, ireqs):
             key = key_from_ireq(ireq)
-            if key in upgrade_install_reqs:
-                existing_pins_to_upgrade.add(key)
-            else:
+            if key not in upgrade_install_reqs:
                 existing_pins[key] = ireq
         repository = LocalRequirementsRepository(
             existing_pins, repository, reuse_hashes=reuse_hashes
@@ -507,6 +510,27 @@ def cli(
                 )
             )
 
+    if upgrade_packages:
+        constraints_file = tempfile.NamedTemporaryFile(mode="wt", delete=False)
+        constraints_file.write("\n".join(upgrade_packages))
+        constraints_file.flush()
+        try:
+            reqs = list(
+                parse_requirements(
+                    constraints_file.name,
+                    finder=repository.finder,
+                    session=repository.session,
+                    options=repository.options,
+                    constraint=True,
+                )
+            )
+        finally:
+            constraints_file.close()
+            os.unlink(constraints_file.name)
+        for req in reqs:
+            req.comes_from = None
+        constraints.extend(reqs)
+
     extras = tuple(itertools.chain.from_iterable(ex.split(",") for ex in extras))
 
     if extras and not setup_file_found:
@@ -517,19 +541,21 @@ def cli(
         key_from_ireq(ireq) for ireq in constraints if not ireq.constraint
     }
 
-    allowed_upgrades = primary_packages | existing_pins_to_upgrade
     constraints.extend(
-        ireq for key, ireq in upgrade_install_reqs.items() if key in allowed_upgrades
+        ireq for key, ireq in upgrade_install_reqs.items() if key in primary_packages
     )
 
     constraints = [req for req in constraints if req.match_markers(extras)]
     for req in constraints:
         drop_extras(req)
 
-    log.debug("Using indexes:")
-    with log.indentation():
-        for index_url in dedup(repository.finder.index_urls):
-            log.debug(redact_auth_from_url(index_url))
+    if repository.finder.index_urls:
+        log.debug("Using indexes:")
+        with log.indentation():
+            for index_url in dedup(repository.finder.index_urls):
+                log.debug(redact_auth_from_url(index_url))
+    else:
+        log.debug("Ignoring indexes.")
 
     if repository.finder.find_links:
         log.debug("")
@@ -599,6 +625,7 @@ def cli(
     )
     writer.write(
         results=results,
+        unsafe_packages=resolver.unsafe_packages,
         unsafe_requirements=resolver.unsafe_constraints,
         markers={
             key_from_ireq(ireq): ireq.markers for ireq in constraints if ireq.markers
